/**
   A SymbolTable holds <variable,value> pairs.

*/
import java.util.ArrayList;

public class SymbolTable
{
   private ArrayList<String> variables;
   private ArrayList<Value>  values;

   // Default constructor
   public SymbolTable()
   {
      variables = new ArrayList<String>();
      values    = new ArrayList<Value>();
   }

   /**
      Add a <variable, value> pair to this symbol table.
   */
   public void add(String variable, Value value)
   {
      variables.add(variable);
         values.add(value);
   }

   /**
      Look up variable in this symbol table and
      return its associated value.

      Throws an Error if variable is not found.
   */
   public Value lookUp(String variable)
   {
      int i;
      for (i=0; i < variables.size(); i++)
         if (variable.equals(variables.get(i)))
            break;

      if ( i < variables.size() )
         return values.get(i);
      else
         throw new Error("undefined variable: " + variable);
   }

   /**
      Look up variable in this symbol table.
      Return true if the variable is in the table,
      otherwise return false.
   */
   public boolean defined(String variable)
   {
      int i;
      for (i=0; i < variables.size(); i++)
         if (variable.equals(variables.get(i)))
            break;

      if ( i < variables.size() )
         return true;
      else
         return false;
   }


   /**
      Update the value associated with variable.
      Return true if the update is succesfull.
      Return false if variable is not found in the table.
   */
   public boolean update(String variable, Value value)
   {
      int i;
      for (i=0; i < variables.size(); i++)
         if (variable.equals(variables.get(i)))
            break;

      if ( i < variables.size() )
      {
         values.set(i, value);
         return true;
      }
      else
         return false;
   }


   /**
      Convert the contents of this symbol table into a string.
      This is mainly for debugging purposes.
   */
   public String toString()
   {
      String result = "[Symbol Table\n";
      for (int i = 0; i < variables.size(); i++)
      {
         result += " " + variables.get(i) + " = " + values.get(i) + "\n";
      }
      result += "]";
      return result;
   }

}//SymbolTable